@@ -13,13 +13,15 @@ import re
 from collections import Counter
 from copy import deepcopy
 from functools import lru_cache, partial
-from lxml import etree, html
+
+from lxml.html import tostring
 
 # own
 from .extractors import (discard_unwanted, extract_url_date,
                          extract_partial_url_date, idiosyncrasies_search,
                          img_search, json_search, timestamp_search, try_ymd_date,
-                         ADDITIONAL_EXPRESSIONS, DATE_EXPRESSIONS,
+                         DATE_EXPRESSIONS, FAST_PREPEND, SLOW_PREPEND,
+                         FREE_TEXT_EXPRESSIONS,
                          YEAR_PATTERN, YMD_PATTERN, COPYRIGHT_PATTERN,
                          THREE_PATTERN, THREE_CATCH,
                          THREE_LOOSE_PATTERN, THREE_LOOSE_CATCH,
@@ -40,7 +42,7 @@ from .validators import (check_extracted_reference, compare_values,
 LOGGER = logging.getLogger(__name__)
 def logstring(element):
     '''Format the element to be logged to a string.'''
-    return html.tostring(element, pretty_print=False, encoding='unicode').strip()
+    return tostring(element, pretty_print=False, encoding='unicode').strip()
 
 DATE_ATTRIBUTES = {
                   'article.created', 'article_date_original',
@@ -74,7 +76,9 @@ NON_DIGITS_REGEX = re.compile(r'\D+$')
 GER_STRIP_REGEX = re.compile(r'^am ')
 
 LAST_MODIFIED = {'lastmodified', 'last-modified'}
-ITEMPROP_ATTRS = {'datecreated', 'datepublished', 'pubyear'}
+ITEMPROP_ATTRS_ORIGINAL = {'datecreated', 'datepublished', 'pubyear'}
+ITEMPROP_ATTRS_MODIFIED = {'datemodified', 'dateupdate'}
+ITEMPROP_ATTRS = ITEMPROP_ATTRS_ORIGINAL.union(ITEMPROP_ATTRS_MODIFIED)
 CLASS_ATTRS = {'date-published', 'published', 'time published'}
 
 
@@ -83,7 +87,7 @@ def examine_date_elements(tree, expression, outputformat, extensive_search, min_
     """Check HTML elements one by one for date expressions"""
     try:
         elements = tree.xpath(expression)
-    except etree.XPathEvalError as err:
+    except Exception as err:
         LOGGER.error('lxml expression %s throws an error: %s', expression, err)
         return None
     if not elements or len(elements) > MAX_POSSIBLE_CANDIDATES:
@@ -101,7 +105,7 @@ def examine_date_elements(tree, expression, outputformat, extensive_search, min_
             # more than 4 digits required # list(filter(str.isdigit, toexamine))
             #if len([c for c in toexamine if c.isdigit()]) < 4:
             #    continue
-            LOGGER.debug('analyzing (HTML): %s', html.tostring(
+            LOGGER.debug('analyzing (HTML): %s', tostring(
                 elem, pretty_print=False, encoding='unicode').translate(
                     {ord(c): None for c in '\n\t\r'}
                 ).strip()[:100])
@@ -190,16 +194,21 @@ def examine_header(tree, outputformat, extensive_search, original_date, min_date
         elif 'itemprop' in elem.attrib:
             attribute = elem.get('itemprop').lower()
             # original: store / updated: override date
-            if (
-                attribute in ITEMPROP_ATTRS
-                or (attribute == 'datemodified'
-                and original_date is False)
-            ):
+            if attribute in ITEMPROP_ATTRS:
+                attempt = None
                 LOGGER.debug('examining meta itemprop: %s', logstring(elem))
                 if 'datetime' in elem.attrib:
-                    headerdate = try_ymd_date(elem.get('datetime'), outputformat, extensive_search, min_date, max_date)
+                    attempt = tryfunc(elem.get('datetime'))
                 elif 'content' in elem.attrib:
-                    headerdate = tryfunc(elem.get('content'))
+                    attempt = tryfunc(elem.get('content'))
+                # store value
+                if attempt is not None:
+                    if (attribute in ITEMPROP_ATTRS_ORIGINAL and original_date is True) or \
+                       (attribute in ITEMPROP_ATTRS_MODIFIED and original_date is False):
+                        headerdate = attempt
+                    # put on hold: hurts precision
+                    #else:
+                    #    reserve = attempt
             # reserve with copyrightyear
             elif attribute == 'copyrightyear':
                 LOGGER.debug('examining meta itemprop: %s', logstring(elem))
@@ -602,8 +611,8 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
     if url is None:
         # link canonical
         for elem in tree.iterfind('.//link[@rel="canonical"]'):
-            if 'href' in elem.attrib:
-                url = elem.get('href')
+            url = elem.get('href')
+            if url is not None:
                 break
     if url is not None:
         dateresult = extract_url_date(url, outputformat)
@@ -629,25 +638,24 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
         return abbr_result
 
     # expressions + text_content
+    if extensive_search is True:
+        date_expr = SLOW_PREPEND + DATE_EXPRESSIONS
+    else:
+        date_expr = FAST_PREPEND + DATE_EXPRESSIONS
     # first try in pruned tree
     search_tree, discarded = discard_unwanted(deepcopy(tree))
-    dateresult = examine_date_elements(search_tree, DATE_EXPRESSIONS,
+    dateresult = examine_date_elements(search_tree, date_expr,
         outputformat, extensive_search, min_date, max_date)
     if dateresult is not None:
         return dateresult
-    # supply more expressions (other languages)
-    if extensive_search is True:
-        dateresult = examine_date_elements(search_tree, ADDITIONAL_EXPRESSIONS,
-            outputformat, extensive_search, min_date, max_date)
-        if dateresult is not None:
-            return dateresult
 
-    # search in discarded parts (currently: footers and archive.org-banner)
-    for subtree in discarded:
-        dateresult = examine_date_elements(subtree, DATE_EXPRESSIONS,
-            outputformat, extensive_search, min_date, max_date)
-        if dateresult is not None:
-            return dateresult
+    # TODO: decide on this
+    # search in discarded parts (e.g. archive.org-banner)
+    #for subtree in discarded:
+    #    dateresult = examine_date_elements(subtree, DATE_EXPRESSIONS,
+    #        outputformat, extensive_search, min_date, max_date)
+    #    if dateresult is not None:
+    #        return dateresult
 
     # try time elements
     time_result = examine_time_elements(
@@ -658,15 +666,15 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
 
     # clean before string search
     try:
-        cleaned_html = HTML_CLEANER.clean_html(deepcopy(tree))
+        cleaned_html = HTML_CLEANER.clean_html(tree)
     # rare LXML error: no NULL bytes or control characters
     except ValueError:
         cleaned_html = tree
     # robust conversion to string
     try:
-        htmlstring = html.tostring(cleaned_html, pretty_print=False, encoding='unicode')
+        htmlstring = tostring(cleaned_html, pretty_print=False, encoding='unicode')
     except UnicodeDecodeError:
-        htmlstring = html.tostring(cleaned_html, pretty_print=False).decode('utf-8', 'ignore')
+        htmlstring = tostring(cleaned_html, pretty_print=False).decode('utf-8', 'ignore')
     # remove comments by hand as faulty in lxml?
     # htmlstring = re.sub(r'<!--.+?-->', '', htmlstring, flags=re.DOTALL)
 
@@ -702,11 +710,10 @@ def find_date(htmlobject, extensive_search=True, original_date=False, outputform
     # last resort
     if extensive_search is True:
         LOGGER.debug('extensive search started')
-        # div and p elements?
-        # TODO: check all and decide according to original_date
+        # TODO: further tests & decide according to original_date
         reference = 0
-        for textpart in [t for t in cleaned_html.xpath('.//div/text()|.//p/text()') if 0 < len(t) < 80]:
-            reference = compare_reference(reference, textpart, outputformat, extensive_search, original_date, min_date, max_date)
+        for segment in [t for t in cleaned_html.xpath(FREE_TEXT_EXPRESSIONS) if 6 < len(t) < 60]:
+            reference = compare_reference(reference, segment, outputformat, extensive_search, original_date, min_date, max_date)
         # return
         converted = check_extracted_reference(reference, outputformat, min_date, max_date)
         if converted is not None:
