@@ -11,12 +11,12 @@ import os
 import re
 import sys
 
-import pytest
-
 from collections import Counter
 from contextlib import redirect_stdout
 from unittest.mock import Mock, patch
 
+import pytest
+
 try:
     import dateparser
     EXT_PARSER = True
@@ -28,12 +28,11 @@ from lxml import html
 
 from htmldate.cli import examine, main, parse_args, process_args
 from htmldate.core import compare_reference, examine_date_elements, find_date, search_page, search_pattern, select_candidate, try_ymd_date
-from htmldate.extractors import custom_parse, external_date_parser, extract_partial_url_date, regex_parse
+from htmldate.extractors import DATE_EXPRESSIONS, custom_parse, external_date_parser, extract_partial_url_date, regex_parse
 from htmldate.settings import MIN_DATE, LATEST_POSSIBLE
 from htmldate.utils import decode_response, detect_encoding, fetch_url, load_html, is_dubious_html
 from htmldate.validators import convert_date, date_validator, get_max_date, get_min_date, output_format_validator
 
-from urllib3 import HTTPResponse
 
 TEST_DIR = os.path.abspath(os.path.dirname(__file__))
 OUTPUTFORMAT = '%Y-%m-%d'
@@ -131,8 +130,9 @@ def test_input():
     '''test if loaded strings/trees are handled properly'''
     assert is_dubious_html('This is a string.') is True
     assert is_dubious_html(b'This is a string.') is True
-    with pytest.raises(TypeError):
+    with pytest.raises(TypeError) as err:
         assert load_html(123) is None
+    assert 'incompatible' in str(err.value)
     assert load_html('<'*100) is None
     assert load_html('<html><body>XYZ</body></html>') is not None
     assert load_html(b'<html><body>XYZ</body></html>') is not None
@@ -150,7 +150,9 @@ def test_input():
     # find_date logic
     with pytest.raises(TypeError):
         assert find_date(None) is None
+    assert find_date('<'*100) is None
     assert find_date('<html></html>', verbose=True) is None
+    assert find_date('<html><body>\u0008this\xdf\n\u001f+\uffff</body></html>') is None
     # min and max date output
     assert get_min_date('2020-02-20') == datetime.date(2020, 2, 20)
     assert get_min_date(None) == datetime.date(1995, 1, 1)
@@ -164,9 +166,9 @@ def test_sanity():
     '''Test if function arguments are interpreted and processed correctly.'''
     # XPath looking for date elements
     mytree = html.fromstring('<html><body><p>Test.</p></body></html>')
-    result = examine_date_elements(mytree, '//[Error', True, OUTPUTFORMAT, MIN_DATE, LATEST_POSSIBLE)
+    result = examine_date_elements(mytree, '//[Error', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE)
     assert result is None
-    result = examine_date_elements(mytree, '//p', True, OUTPUTFORMAT, MIN_DATE, LATEST_POSSIBLE)
+    result = examine_date_elements(mytree, '//p', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE)
     assert result is None
     # wrong field values in output format
     assert output_format_validator('%Y-%m-%d') is True
@@ -174,8 +176,6 @@ def test_sanity():
     assert output_format_validator('ABC') is False
     assert output_format_validator(123) is False
     #assert output_format_validator('%\xaa') is False
-    # select_candidate()
-    # ...
 
 
 
@@ -226,6 +226,8 @@ def test_exact_date():
     assert find_date('<html><head><meta pubdate="pubDate" content="2018-02-06"/></head><body></body></html>') == '2018-02-06'
     assert find_date('<html><head><meta itemprop="DateModified" datetime="2018-02-06"/></head><body></body></html>') == '2018-02-06'
     assert find_date('<html><head><meta name="DC.Issued" content="2021-07-13"/></head><body></body></html>') == '2021-07-13'
+    assert find_date('<html><head><meta itemprop="dateUpdate" datetime="2018-02-06"/></head><body></body></html>', original_date=True) == '2018-02-06'
+    assert find_date('<html><head><meta itemprop="dateUpdate" datetime="2018-02-06"/></head><body></body></html>', original_date=False) == '2018-02-06'
 
     ## time in document body
     assert find_date(load_mock_page('https://www.facebook.com/visitaustria/'), original_date=True) == '2017-10-06'
@@ -289,8 +291,8 @@ def test_exact_date():
     assert find_date(load_mock_page('http://www.stuttgart.de/')) == '2017-10-09'
 
     ## in document body
-    assert find_date(load_mock_page('https://github.com/adbar/htmldate')) == '2019-01-01'
-    #assert find_date(load_mock_page('https://github.com/adbar/htmldate'), original_date=True) == '2016-07-12'
+    #assert find_date(load_mock_page('https://github.com/adbar/htmldate')) == '2019-01-01'
+    assert find_date(load_mock_page('https://github.com/adbar/htmldate'), original_date=True) == '2016-07-12'
 
     assert find_date(load_mock_page('https://en.blog.wordpress.com/')) == '2017-08-30'
     assert find_date(load_mock_page('https://www.austria.info/')) == '2017-09-07'
@@ -354,7 +356,6 @@ def test_exact_date():
     # in link title
     assert find_date('<html><body><a class="ribbon date " title="12th December 2018" href="https://example.org/" itemprop="url">Text</a></body></html>') == '2018-12-12'
 
-
     # archive.org documents
     assert find_date(load_mock_page('http://web.archive.org/web/20210916140120/https://www.kath.ch/die-insel-der-klosterzoeglinge/'), extensive_search=False) is None
     assert find_date(load_mock_page('http://web.archive.org/web/20210916140120/https://www.kath.ch/die-insel-der-klosterzoeglinge/'), extensive_search=True) == '2021-07-13'
@@ -422,6 +423,8 @@ def test_try_ymd_date():
     assert try_ymd_date('12:00 h', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is None
     # date range
     assert try_ymd_date('2005-2006', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) is None
+    # Mandarin
+    assert try_ymd_date('发布时间: 2022-02-25 14:34', OUTPUTFORMAT, True, MIN_DATE, LATEST_POSSIBLE) == '2022-02-25'
 
 
 # def test_header():
@@ -699,6 +702,12 @@ def test_search_html(original_date=False, min_date=MIN_DATE, max_date=LATEST_POS
 def test_idiosyncrasies():
     assert find_date('<html><body><p><em>Last updated: 5/5/20</em></p></body></html>') == '2020-05-05'
     assert find_date('<html><body><p><em>Last updated: 01/23/2021</em></p></body></html>') == '2021-01-23'
+    assert find_date('<html><body><p><em>Last updated: 01/23/21</em></p></body></html>') == '2021-01-23'
+    assert find_date('<html><body><p><em>Last updated: 1/23/21</em></p></body></html>') == '2021-01-23'
+    assert find_date('<html><body><p><em>Last updated: 23/1/21</em></p></body></html>') == '2021-01-23'
+    assert find_date('<html><body><p><em>Last updated: 23/01/21</em></p></body></html>') == '2021-01-23'
+    assert find_date('<html><body><p><em>Last updated: 23/01/2021</em></p></body></html>') == '2021-01-23'
+    assert find_date('<html><body><p><em>Last updated: 33/23/3033</em></p></body></html>') is None
     assert find_date('<html><body><p><em>Published: 5/5/2020</em></p></body></html>') == '2020-05-05'
     assert find_date('<html><body><p><em>Published in: 05.05.2020</em></p></body></html>') == '2020-05-05'
     assert find_date('<html><body><p><em>Son güncelleme: 5/5/20</em></p></body></html>') == '2020-05-05'
